/**
 * iModal for Bootstrap 5
 * Version: 1.0.0.a-1
 * Autor: gusma62@gmail.com
 * https://github.com/GuxMartin/iModal
 */


/**
 * http://youmightnotneedjquery.com/#extend
 */
let extend = function(out) {
 out = out || {};

 for (var i = 1; i < arguments.length; i++) {
   if (!arguments[i])
   continue;

   for (var key in arguments[i]) {
     if (arguments[i].hasOwnProperty(key))
     out[key] = arguments[i][key];
   }
 }

 return out;
};


window.iModalDefaults = {
  id: "iModal",
  size: "",
  fullscreen: false,
  title: "",
  closeText: "Close",
  body: `
    <div class="d-flex justify-content-center" style="height: 100%;">
      <div class="spinner-border text-primary my-auto" role="status">
        <span class="visually-hidden">Loading...</span>
      </div>
    </div>
  `,
  headerClass: "",
  footerClass: "",
  header: true,
  footer: true,
  backdrop: true,
  backdropColor: false,
  keyboard: true,
  dialogScrollable: false,
  dialogCentered: false,
  fade: true,
  onShow: function(){},
  onShowed: function(){},
  onHide: function(){},
  onHidden: function(){},
  onHidePrevented: function(){},
};


window.iModalSettings = {};


window.iModal = function(params){

  if(typeof(params) == 'string'){
    let el_iModal = document.querySelector('.modal.iModal');
    let iModalInstance = null;
    if(el_iModal){
      iModalInstance = bootstrap.Modal.getInstance(el_iModal);
    }

    switch(params){
      case 'isActive':
        return !! el_iModal;
      break;// /isActive
      case 'remove':
        if(! iModal('isActive')){ return; }
        // modal.hide();
        iModalInstance.dispose();
        el_iModal.parentNode.removeChild(el_iModal);
        document.querySelector('body').classList.remove("modal-open");
        let modal_backdrop = document.querySelector('.modal-backdrop');
        if(modal_backdrop){
          modal_backdrop.parentNode.removeChild(modal_backdrop);
        }
      break;// /remove
      case 'hide':
        if(! iModal('isActive')){ return; }
        iModalInstance.hide();
      break;// /hide
      case 'version':
        return "1.0.0.a-1";
      break;// /version
      default:
        console.log("Bad option");
      break;
    }// /switch params
    return;
  }// /typeof(params) == 'string'

  var settings = extend(iModalDefaults, iModalSettings, params);

  // Array de clases de colores de fondo de bootstrap
  let bgColors = ['success','primary','info','warning','danger','secondary'];

  // Comprobación de parámetros
  if(settings.id.length < 1){ settings.id = iModalDefaults.id; }

  // TODO: fullscreen-sm-down, etc..
  if(['sm','lg','xl'].indexOf(settings.size) < 0){ settings.size = iModalDefaults.size; }
  else{ settings.size = 'modal-'+settings.size; }

  if([false,true,'sm','md','lg','xl','xxl'].indexOf(settings.fullscreen) < 0){ settings.fullscreen = iModalDefaults.fullscreen; }
  else{
    if(settings.fullscreen !== false){
      if(settings.fullscreen === true){ settings.fullscreen = 'modal-fullscreen'; }
      else {
        settings.fullscreen = `modal-fullscreen-${settings.fullscreen}-down`;
      }
    }
  }
  console.log(settings.fullscreen);

  if(bgColors.indexOf(settings.headerClass) < 0){ settings.headerClass = iModalDefaults.headerClass; }
  else{ settings.headerClass = 'bg-'+settings.headerClass; }

  if(bgColors.indexOf(settings.footerClass) < 0){ settings.footerClass = iModalDefaults.footerClass; }
  else{ settings.footerClass = 'bg-'+settings.footerClass; }

  if([true,false,'static'].indexOf(settings.backdrop) < 0){ settings.backdrop = iModalDefaults.backdrop; }

  if(bgColors.indexOf(settings.backdropColor) < 0){ settings.backdropColor = iModalDefaults.backdropColor; }
  else{ settings.backdropColor = 'bg-'+settings.backdropColor; }

  if([true,false].indexOf(settings.keyboard) < 0){ settings.keyboard = iModalDefaults.keyboard; }

  if([true,false].indexOf(settings.dialogScrollable) < 0){ settings.dialogScrollable = iModalDefaults.dialogScrollable; }

  if([true,false].indexOf(settings.dialogCentered) < 0){ settings.dialogCentered = iModalDefaults.dialogCentered; }

  if([true,false].indexOf(settings.fade) < 0){ settings.fade = iModalDefaults.fade; }

  // Por ver
  // if(settings.keyboard === false){
  //   settings.backdrop = 'static';// Obligatorio static https://getbootstrap.com/docs/4.5/components/modal/#static-backdrop
  // }

  // Si ya existiera, destruir para regenerar
  iModal('remove');

  let modal = document.createElement('div');
  modal.id = settings.id;
  modal.tabIndex = -1;
  modal.classList.add("modal", "iModal");
  if(settings.fade){
    modal.classList.add("fade");
  }

  let modal_dialog = document.createElement('div');
  modal_dialog.classList.add("modal-dialog");
  if(settings.size){
    modal_dialog.classList.add(settings.size);
  }
  if(settings.fullscreen){
    modal_dialog.classList.add(settings.fullscreen);
  }
  if(settings.dialogScrollable){
    modal_dialog.classList.add("modal-dialog-scrollable");
  }
  if(settings.dialogCentered){
    modal_dialog.classList.add("modal-dialog-centered");
  }
  modal.appendChild(modal_dialog);

  let modal_content = document.createElement('div');
  modal_content.classList.add("modal-content");
  modal_dialog.appendChild(modal_content);

  if(settings.header){
    let modal_header = document.createElement('div');
    modal_header.classList.add("modal-header");
    if(settings.headerClass){
      modal_header.classList.add(settings.headerClass);
      modal_header.classList.add('text-white');
    }
    modal_header.innerHTML = `
      <h5 class="modal-title">${settings.title}</h5>
      <button type="button" class="btn-close" data-bs-dismiss="modal" aria-label="Close"></button>
    `;
    modal_content.appendChild(modal_header);
  }

  let modal_body = document.createElement('div');
  modal_body.classList.add("modal-body");
  modal_body.innerHTML = settings.body;
  modal_content.appendChild(modal_body);

  if(settings.footer){
    let modal_footer = document.createElement('div');
    modal_footer.classList.add("modal-footer");
    if(settings.footerClass){
      modal_footer.classList.add(settings.footerClass);
    }
    modal_footer.innerHTML = `
      <button type="button" class="btn btn-secondary" data-bs-dismiss="modal">${settings.closeText}</button>
    `;
    modal_content.appendChild(modal_footer);
  }

  let document_body = document.querySelector('body');
  document_body.appendChild(modal);


  let iModalId = document.getElementById(settings.id);


  // Declarar events antes de mostrar
  iModalId.addEventListener('show.bs.modal', function (event) {
    settings.onShow();
  });
  iModalId.addEventListener('shown.bs.modal', function (event) {
    settings.onShowed();
  });
  iModalId.addEventListener('hide.bs.modal', function (event) {
    settings.onHide();
  });
  iModalId.addEventListener('hidden.bs.modal', function (event) {
    settings.onHidden();
    iModal('remove');
  });
  iModalId.addEventListener('hidePrevented.bs.modal', function (event) {
    settings.onHidePrevented();
  });


  // Generar modal y mostrar
  let myiModal = new bootstrap.Modal(iModalId, {
    backdrop: settings.backdrop,
    keyboard: settings.keyboard,
  });

  myiModal.show();

  // Backdrop Color
  if(settings.backdropColor){
    document.querySelector('.modal-backdrop').classList.add(settings.backdropColor);
  }

};
